﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace SampleExternalServiceAPI.BL
{
    public class TaxCalculatorManager
    {
        private TaxSummary _taxSummary;
        private decimal _taxPercentage;
        private bool _validateManufacture;
        private bool _validatePickupAddress;
        private bool _validateTaxCode;


        public TaxCalculatorManager(decimal taxPercentage, bool validateManufacture, bool validatePickupAddress, bool validateTaxCode)
        {
            _taxSummary = new TaxSummary();
            _taxPercentage = taxPercentage;
            _validateManufacture = validateManufacture;
            _validatePickupAddress = validatePickupAddress;
            _validateTaxCode = validateTaxCode;
        }

        public void CalculateTax(JObject calculationDetails)
        {

            JArray orderProductsArray = (JArray)calculationDetails["Order"]["OrderProducts"];
            decimal optionalOrderShippingPrice = (calculationDetails["Order"]["Prices"]["ShippingPrice"] != null) ? decimal.Parse(calculationDetails["Order"]["Prices"]["ShippingPrice"].ToString()) : 0;

            validateOrderPickingAddress((JArray)calculationDetails["Order"]["Deliveries"]);

       
            calculateAllOrderProducts(orderProductsArray, optionalOrderShippingPrice);
        }

        private void validateOrderPickingAddress(JArray orderDeliveriesArray)
        {
            if (_validatePickupAddress)
            {
                foreach (JObject item in orderDeliveriesArray)
                {
                    if (item["ShippingAddress"] != null && item["ShippingAddress"].HasValues)
                    {
                        if (item["ShippingAddress"]["IsPickupAddress"] == null)
                        {
                            throw new HttpException(400, "IsPickupAdress is missing in Order Deliveries");
                        }
                    }
                }
            }
        }


        private void validateTaxData(JObject orderProduct, bool isSecondIterate)
        {
            JToken taxGroup = orderProduct["Product"]["TaxGroup"];
            if (_validateTaxCode)
            {
                if (taxGroup == null || (taxGroup!= null && !taxGroup.HasValues)
                    || taxGroup["Id"] == null || (taxGroup["Id"] != null && taxGroup["Id"].Type == JTokenType.Null)
                    || taxGroup["Name"] == null || (taxGroup["Name"] != null && taxGroup["Name"].Type == JTokenType.Null)
                    || taxGroup["TaxCode"] == null || (taxGroup["TaxCode"] != null && taxGroup["TaxCode"].Type == JTokenType.Null))
                {
                    throw new HttpException(400, $"Tax data is missing inside product ID: {orderProduct["Product"]["@id"].ToString()}");
                }

                if (!isSecondIterate)
                {
                    JArray subOrderProductsArray = orderProduct.GetValue("SubOrderProducts") != null ? (JArray)orderProduct.GetValue("SubOrderProducts") : null;
                    if (subOrderProductsArray != null)
                    {
                        foreach (JObject subItem in subOrderProductsArray)
                        {
                            validateTaxData(subItem, true);
                        }
                    }
                }
            }
        }

        private void validateManufacturerData(JObject orderProduct, bool isSecondIterate)
        {
            if (_validateManufacture)
            {
                if (orderProduct["Product"]["Manufacturer"] == null || orderProduct["Product"]["Manufacturer"]["Address"] == null)
                {
                    throw new HttpException(400, $"Manufacturer data is missing inside product ID: {orderProduct["Product"]["@id"].ToString()}");
                }

                if (!isSecondIterate)
                {
                    JArray subOrderProductsArray = orderProduct.GetValue("SubOrderProducts") != null ? (JArray)orderProduct.GetValue("SubOrderProducts") : null;
                    if (subOrderProductsArray != null)
                    {
                        foreach (JObject subItem in subOrderProductsArray)
                        {
                            validateManufacturerData(subItem, true);
                        }
                    }
                }
            }
        }

        public string GetMinimalJsonResult()
        {
            return  $"{{\"TotalTax\":\"{_taxSummary.TotalTax}\"}}";
        }

        public string GetFullJsonResult()
        {
            TaxSummaryJsonToSerilize taxSumaryToSeriliaze = new TaxSummaryJsonToSerilize();
            taxSumaryToSeriliaze.TotalTax = _taxSummary.TotalTax;
            taxSumaryToSeriliaze.Taxes = _taxSummary.TaxFullDictionary.Values.ToList();
            taxSumaryToSeriliaze.TaxItems = _taxSummary.TaxItemsList;

            string response = JsonConvert.SerializeObject(taxSumaryToSeriliaze);

            return response;
        }


        private void calculateAllOrderProducts(JArray orderProductsArray, decimal optionalOrderShippingPrice)
        {
            string taxGroupID;
            TaxItem tempTaxItem;
            TaxFull tempTaxFull;
            decimal tempCalculatedTax;

            if (orderProductsArray == null)
                return;

            decimal addedShippingPricePerOrderProduct = optionalOrderShippingPrice / orderProductsArray.Count();
            decimal tempNewAmount;
            foreach (JObject item in orderProductsArray)
            {
                validateTaxData(item, false);
                validateManufacturerData(item, false);
                tempNewAmount = decimal.Parse(item.GetValue("Prices")["Subtotal"].ToString()) + addedShippingPricePerOrderProduct;
                tempTaxItem = new TaxItem(int.Parse(item.GetValue("@id").ToString()),
                                         tempNewAmount);
                
                taxGroupID = item.GetValue("Product")["TaxGroup"]["Id"].ToString();
                tempCalculatedTax = tempTaxItem.AddTaxGetCalcuateAmount(int.Parse(taxGroupID), _taxPercentage);

                _taxSummary.TaxItemsList.Add(tempTaxItem);

                if (_taxSummary.TaxFullDictionary.ContainsKey(taxGroupID))
                {
                    _taxSummary.TaxFullDictionary[taxGroupID].TaxAmount += tempCalculatedTax;
                }
                else
                {
                    tempTaxFull = new TaxFull();
                    tempTaxFull.TaxID = int.Parse(taxGroupID);
                    tempTaxFull.TaxAmount = tempCalculatedTax;
                    tempTaxFull.Percentage = _taxPercentage * 100;
                    tempTaxFull.Name = item.GetValue("Product")["TaxGroup"]["Name"].ToString();
                    tempTaxFull.DisplayName = "Display_" + tempTaxFull.Name;

                    _taxSummary.TaxFullDictionary.Add(taxGroupID, tempTaxFull);
                }
                _taxSummary.TotalTax += tempCalculatedTax;
            }
        }
    }
}